import numpy as np
import atomwrite
import yag
import pickle as pickle
import pylab as plt
import time


def InitYAGlattice(cellparam,(x,y,z),numCE=4,UCa=1,UCb=1,UCc=1):
    '''
initializes the YAG crystalline lattice positions, returns positions and atomic identities
inputs:
    cellparam: unit cell parameter (cubic cell so you only need one)
    (x,y,z): oxygen xyz position (Y and Al have special fixed positions)
    CEconc: concentration of cerium dopant ions in the lattice
    UCa,UCb,UCc: number of unit cells in a,b,c directions, default 1 total unit cell
outputs:
    Pos: (N,3) array atomic positions
    AtomIDs: (N) array of atomic identities corresponding to each Pos. IDs are: octAl=1, Y=2, tetAl=3, oxygen=4, Ce=5
    atom_names: (N) array of atom names as strings for chimera
    '''

    ######### Symmetry operations for Ia-3d space group (#230) from the international tables of crystallography ########
    site16a_operations0=np.array([(0, 0, 0), (1/2., 0, 1/2.), (0, 1/2., 1/2.), (1/2., 1/2., 0), (3/4., 1/4., 1/4.),
    (3/4., 3/4., 3/4.), (1/4., 1/4., 3/4.), (1/4., 3/4., 1/4.)])
    site16a_operations05=site16a_operations0+0.5
    site16a_operations=np.concatenate((site16a_operations0,site16a_operations05))
    for i in range(len(site16a_operations)):
        for j in range(3):
            if site16a_operations[i,j]>1.:
                site16a_operations[i,j]=site16a_operations[i,j]-1
    
    site24c_operations0=np.array([(1/8., 0, 1/4.), (3/8., 0, 3/4.), (1/4., 1/8., 0), (3/4., 3/8., 0), (0, 1/4., 1/8.),
    (0, 3/4., 3/8.), (7/8., 0, 3/4.), (5/8., 0, 1/4.), (3/4., 7/8., 0), (1/4., 5/8., 0), (0, 3/4., 7/8.), (0, 1/4., 5/8.)])
    site24c_operations05=site24c_operations0+0.5
    site24c_operations=np.concatenate((site24c_operations0,site24c_operations05))
    for i in range(len(site24c_operations)):
        for j in range(3):
            if site24c_operations[i,j]>1.:
                site24c_operations[i,j]=site24c_operations[i,j]-1
    
    site24d_operations0=np.array([(3/8., 0, 1/4.), (1/8., 0, 3/4.), (1/4., 3/8., 0), (3/4., 1/8., 0), (0, 1/4., 3/8.),
    (0, 3/4., 1/8.), (3/4., 5/8., 0), (3/4., 3/8., 1/2.), (1/8., 1/2., 1/4.), (7/8., 0, 1/4.), (0, 1/4., 7/8.), (1/2., 1/4., 1/8.)])
    site24d_operations05=site24d_operations0+0.5
    site24d_operations=np.concatenate((site24d_operations0,site24d_operations05))
    for i in range(len(site24d_operations)):
        for j in range(3):
            if site24d_operations[i,j]>1.:
                site24d_operations[i,j]=site24d_operations[i,j]-1
    
    site96h_operations0=np.array([(x, y, z), (-x + 1/2., -y, z + 1/2.), (-x, y + 1/2., -z + 1/2.), (x + 1/2., -y + 1/2., -z),
    (z, x, y), (z + 1/2., -x + 1/2., -y), (-z + 1/2., -x, y + 1/2.), (-z, x + 1/2., -y + 1/2.), (y, z, x),
    (-y, z + 1/2., -x + 1/2.), (y + 1/2., -z + 1/2., -x), (-y + 1/2., -z, x + 1/2.), (y + 3/4., x + 1/4., -z + 1/4.),
    (-y + 3/4., -x + 3/4., -z + 3/4.), (y + 1/4., -x + 1/4., z + 3/4.), (-y + 1/4., x + 3/4., z + 1/4.),
    (x + 3/4., z + 1/4., -y + 1/4.), (-x + 1/4., z + 3/4., y + 1/4.), (-x + 3/4., -z + 3/4., -y + 3/4.),
    (x + 1/4., -z + 1/4., y + 3/4.), (z + 3/4., y + 1/4., -x + 1/4.), (z + 1/4., -y + 1/4., x + 3/4.),
    (-z + 1/4., y + 3/4., x + 1/4.), (-z + 3/4., -y + 3/4., -x + 3/4.), (-x, -y, -z), (x + 1/2., y, -z + 1/2.),
    (x, -y + 1/2., z + 1/2.), (-x + 1/2., y + 1/2., z), (-z, -x, -y), (-z + 1/2., x + 1/2., y), (z + 1/2., x, -y + 1/2.),
    (z, -x + 1/2., y + 1/2.), (-y, -z, -x), (y, -z + 1/2., x + 1/2.), (-y + 1/2., z + 1/2., x), (y + 1/2., z, -x + 1/2.),
    (-y + 1/4., -x + 3/4., z + 3/4.), (y + 1/4., x + 1/4., z + 1/4.), (-y + 3/4., x + 3/4., -z + 1/4.),
    (y + 3/4., -x + 1/4., -z + 3/4.), (-x + 1/4., -z + 3/4., y + 3/4.), (x + 3/4., -z + 1/4., -y + 3/4.),
    (x + 1/4., z + 1/4., y + 1/4.), (-x + 3/4., z + 3/4., -y + 1/4.), (-z + 1/4., -y + 3/4., x + 3/4.),
    (-z + 3/4., y + 3/4., -x + 1/4.), (z + 3/4., -y + 1/4., -x + 3/4.), (z + 1/4., y + 1/4., x + 1/4.)])

 #### accounting for the symmetry operation +1/2, 1/2, 1/2 ###
    site96h_operations05=site96h_operations0+0.5
    site96h_operations=np.concatenate((site96h_operations0,site96h_operations05))
    for i in range(len(site96h_operations)):  ###### make sure all the O positions are within the unit cell
        for j in range(3):
            if site96h_operations[i,j]>1.:
                site96h_operations[i,j]=site96h_operations[i,j]-1.
            if site96h_operations[i,j]<0:
                site96h_operations[i,j]=site96h_operations[i,j]+1.
                
    ########  octahedral Al is on the 16a site, Y on 24c site, tetrahedral Al on the 24d site, 
    ########  oxygen on 96h, put them in the position array in that order.
    ######## IDs are: octAl=1, Y=2, tetAl=3, oxygen=4, Ce=5                        #####
    
    ####### tiling unit cells according to UCa, UCb, UCc
    site16atemp=site16a_operations
    site16atotal=site16a_operations
    
    site24ctemp=site24c_operations
    site24ctotal=site24c_operations
    
    site24dtemp=site24d_operations
    site24dtotal=site24d_operations
    
    site96htemp=site96h_operations
    site96htotal=site96h_operations
     
    for ashift in range(UCa):             
        for bshift in range(UCb):        
            for cshift in range(UCc):
                site16atemp=site16a_operations+(ashift,bshift,cshift)
                site24ctemp=site24c_operations+(ashift,bshift,cshift)
                site24dtemp=site24d_operations+(ashift,bshift,cshift)
                site96htemp=site96h_operations+(ashift,bshift,cshift)
                
                if (ashift,bshift,cshift)!=(0,0,0):
                    site16atotal=np.concatenate((site16atotal,site16atemp))
                    site24ctotal=np.concatenate((site24ctotal,site24ctemp))
                    site24dtotal=np.concatenate((site24dtotal,site24dtemp))
                    site96htotal=np.concatenate((site96htotal,site96htemp))
        
    site16atotal=site16atotal*cellparam  ##scale positions to unit cell size
    site24ctotal=site24ctotal*cellparam
    site24dtotal=site24dtotal*cellparam
    site96htotal=site96htotal*cellparam
        
    Pos=np.concatenate((site16atotal, site24ctotal, site24dtotal, site96htotal)) ##populate atomic position list
    
    ids24c=np.ones(24*UCa*UCb*UCc)*2  ########## put the ce atom ids in the 24c id set
    sites24c=24*UCa*UCb*UCc
    spacing=sites24c/numCE
    for each in range(numCE):
        ids24c[spacing*each]=5
        
    AtomIDs=np.concatenate((np.ones(16*UCa*UCb*UCc),ids24c,np.ones(24*UCa*UCb*UCc)*3,np.ones(96*UCa*UCb*UCc)*4))
    AtomNames=[]
    AtomMasses=[]
    for i in range(len(AtomIDs)):
        if AtomIDs[i]==1 or AtomIDs[i]==3:
            AtomNames.append('Al')
            AtomMasses.append(26.981538/(1000.*6.022*10.**(23.)))
        if AtomIDs[i]==2:
            AtomNames.append('Y')
            AtomMasses.append(88.90585/(1000.*6.022*10.**(23.)))
        if AtomIDs[i]==4:
            AtomNames.append('O')
            AtomMasses.append(15.9994/(1000.*6.022*10.**(23.)))
        if AtomIDs[i]==5:
            AtomNames.append('Ce')
            AtomMasses.append(140.116/(1000.*6.022*10.**(23.)))
            '''    ###### Perturb the initial positions slightly for non-uniform (non-zero) intial forces
    for i in range(len(Pos)):
        for j in range(3):
            Pos[i,j]+=0.1*(1-2*np.random.random())'''
            
    AtomIDs=np.array(AtomIDs,int)
    AtomMasses=np.array(AtomMasses,float)
    
    return Pos, AtomIDs, AtomNames, AtomMasses


def RescaleVelocities(Vel, T, AtomMasses):
    """Rescales velocities in the system to the target temperature.
Input:
    Vel: (N,3) array of atomic velocities
    T: target temperature
    AtomMasses: (N) array of atomic masses
Output:
    Vel: same as above
"""
    #recenter to zero net momentum
    Vel = Vel - np.average(Vel,axis=0,weights=AtomMasses)
    #find the total kinetic energy
    KE = 0.5  * np.sum(AtomMasses * np.sum((Vel * Vel),axis=1))
    #find velocity scale factor from ratios of kinetic energy
    VScale = np.sqrt(2 * len(Vel) * T * 1.3806503 *10. **(-23.)*10.**10/ KE)
    #print (2 * len(Vel) * T * 1.3806503 *10. **(-23.)*10.**10), KE, VScale
    Vel = Vel * VScale
    return Vel


def InitVelocities(N, T, AtomMasses):
    """Returns an initial random velocity set.
Input:
    N: number of atoms
    T: target temperature
    AtomMasses: (N) array of atomic masses
Output:
    Vel: (N,3) array of atomic velocities
"""
    Vel = np.random.rand(N, 3)
    Vel = RescaleVelocities(Vel, T, AtomMasses)
    return Vel


def InitAccel(Pos,AtomMasses):
    """Returns the initial acceleration array.
Input:
    Pos: (N,3) array of atomic positions
    AtomMasses: (N) array of atomic masses
Output:
    Accel: (N,3) array of acceleration vectors
"""
    #global variables
    global excited, cellparam, rcutbuck, rcutcoul
    #get the acceleration from the forces
    Forces = np.zeros_like(Pos)
    AtomMasses3=np.zeros_like(Pos)
    for i in range(3):
        AtomMasses3[:,i]=AtomMasses
    PEnergy, Forces = yag.calcenergyforces(Pos, AtomIDs, excited, 2*cellparam, rcutbuck, rcutcoul, Forces)
    Accel=Forces/AtomMasses3
    return Accel

UCa=UCb=UCc=1
    
cellparam=12.013
excited=False

rcutbuck=12
rcutcoul=12
starttime=time.time()

numeqsteps=5000
numsteps=30000
numexcsteps=0
numrelaxsteps=0
delt = 10.**(-11.)
T = 300.0
RescaleFreq = 800
savefreq=5000
pdbfreq=5
possavefreq=10
t=range(numeqsteps+numsteps+numexcsteps+numrelaxsteps)
pickle.dump(t,file('t','w'))
t=np.array(t,float)*delt
KE=np.empty_like(t)
PE=np.empty_like(t)
ExcPE=np.empty_like(t)


Pos, AtomIDs, AtomNames, AtomMasses = InitYAGlattice(cellparam,(0.28058,0.10099,0.19946),1,UCa,UCb,UCc)

allpos=Pos.copy()

Vel=InitVelocities(len(Pos),T,AtomMasses)
Accel=InitAccel(Pos,AtomMasses)

Pdb=atomwrite.pdbfile('anim.pdb', UCa*cellparam, AtomNames=AtomNames)
Pdb.write(Pos)

for i in range(numeqsteps):
    if i % RescaleFreq == 0:
        Vel=RescaleVelocities(Vel,T,AtomMasses)
        print float(i)/(numeqsteps+numsteps+numexcsteps+numrelaxsteps)*100, '% done'
    Pos,Vel,Accel,KE[i],PE[i]=yag.vvintegrate(Pos, AtomIDs, Vel, Accel, AtomMasses, UCa*cellparam, False, rcutbuck, rcutcoul, delt*10.)
    ExcPE[i]=yag.calcenergy(Pos, AtomIDs, True, UCa*cellparam, rcutbuck, rcutcoul)
    if i%pdbfreq == 0:
        Pdb.write(Pos)
    if i % savefreq == 0:
        pickle.dump(KE,file('ke','w'))
        pickle.dump(PE,file('pe','w'))
        pickle.dump(ExcPE,file('excpe','w'))
        pickle.dump(allpos,file('allpos','w'))
		
print 'equlibration done in ', (time.time()-starttime)/60, ' mins'

for i in range(numeqsteps,numeqsteps+numsteps):
    if i % (RescaleFreq*10) == 0:
        Vel=RescaleVelocities(Vel,T,AtomMasses)	
        print float(i)/(numeqsteps+numsteps+numexcsteps+numrelaxsteps)*100, '% done'
    Pos,Vel,Accel,KE[i],PE[i]=yag.vvintegrate(Pos, AtomIDs, Vel, Accel, AtomMasses, UCa*cellparam, False, rcutbuck, rcutcoul, delt)
    ExcPE[i]=yag.calcenergy(Pos, AtomIDs, True, UCa*cellparam, rcutbuck, rcutcoul)
    if i%pdbfreq == 0:
        Pdb.write(Pos)
    if i%possavefreq == 0:
        allpos=np.concatenate((allpos,Pos))
    if i % savefreq == 0:
        pickle.dump(KE,file('ke','w'))
        pickle.dump(PE,file('pe','w'))
        pickle.dump(ExcPE,file('excpe','w'))
        pickle.dump(allpos,file('allpos','w'))
        
print 'ground state done in ', (time.time()-starttime)/60, ' mins'        

'''for i in range(numeqsteps+numsteps,numeqsteps+numsteps+numexcsteps):
    if i % RescaleFreq*10 == 0:
        Vel=RescaleVelocities(Vel,T,AtomMasses)
        print float(i)/(numeqsteps+numsteps+numexcsteps+numrelaxsteps)*100, '% done'
    Pos,Vel,Accel,KE[i],ExcPE[i]=yag.vvintegrate(Pos, AtomIDs, Vel, Accel, AtomMasses, UCa*cellparam, True, rcutbuck, rcutcoul, delt)
    PE[i]=yag.calcenergy(Pos, AtomIDs, False, UCa*cellparam, rcutbuck, rcutcoul)
    if i%pdbfreq == 0:
        Pdb.write(Pos)
    if i%possavefreq == 0:
        allpos=np.concatenate((allpos,Pos))
    if i % savefreq == 0:
        pickle.dump(KE,file('ke','w'))
        pickle.dump(PE,file('pe','w'))
        pickle.dump(ExcPE,file('excpe','w'))
        pickle.dump(allpos,file('allpos','w'))

print 'exc state done in ', (time.time()-starttime)/60, ' mins'  

for i in range(numeqsteps+numsteps+numexcsteps,numeqsteps+numsteps+numexcsteps+numrelaxsteps):
    if i % RescaleFreq*10 == 0:
        Vel=RescaleVelocities(Vel,T,AtomMasses)
        print float(i)/(numeqsteps+numsteps+numexcsteps+numrelaxsteps)*100, '% done'
    Pos,Vel,Accel,KE[i],PE[i]=yag.vvintegrate(Pos, AtomIDs, Vel, Accel, AtomMasses, UCa*cellparam, excited, rcutbuck, rcutcoul, delt)
    ExcPE[i]=yag.calcenergy(Pos, AtomIDs, True, UCa*cellparam, rcutbuck, rcutcoul)
    if i%pdbfreq == 0:
        Pdb.write(Pos)
    if i%possavefreq == 0:
        allpos=np.concatenate((allpos,Pos))
    if i % savefreq == 0:
        pickle.dump(KE,file('ke','w'))
        pickle.dump(PE,file('pe','w'))
        pickle.dump(ExcPE,file('excpe','w'))
        pickle.dump(allpos,file('allpos','w'))

print 'full sim done in ', (time.time()-starttime)/60, ' mins'  '''

pickle.dump(KE,file('ke','w'))
pickle.dump(PE,file('pe','w'))
pickle.dump(ExcPE,file('excpe','w'))
pickle.dump(allpos,file('allpos','w'))
pickle.dump(Vel,file('vel','w'))
pickle.dump(Accel,file('accel','w'))

Pdb.close()

print time.time()-starttime

total=PE+KE
#plt.plot(t,total,'red')
plt.plot(t,ExcPE-PE,'blue')
#plt.plot(t,KE,'green')

plt.show()